<?php

namespace Drupal\domain_route_redirect\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

use Drupal\domain\DomainNegotiatorInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\domain_route_redirect\DomainRouteRedirectMatcherService;

use Symfony\Component\HttpKernel\KernelEvents;

use Symfony\Component\HttpKernel\Event\FilterControllerEvent;
use Drupal\Core\Routing\TrustedRedirectResponse;

/**
 * Sets the domain context for an http request.
 */
class DomainRedirectSubscriber implements EventSubscriberInterface {

  /**
   * The domain negotiator service.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface
   */
  protected $domainNegotiator;

  /**
   * The core access check service.
   *
   * @var \Drupal\Core\Access\AccessCheckInterface
   */
  protected $accessCheck;

  /**
   * The current route match.
   *
   * @var Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Constructs a DomainSubscriber object.
   *
   * @param \Drupal\domain\DomainNegotiatorInterface $negotiator
   *   The domain negotiator service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route match.
   * @param Drupal\domain_route_redirect\DomainRouteRedirectMatcherService $matcher
   *   The domain_route_redirect matcher service.
   */
  public function __construct(DomainNegotiatorInterface $negotiator, CurrentRouteMatch $current_route_match, DomainRouteRedirectMatcherService $matcher) {
    $this->domainNegotiator = $negotiator;
    $this->currentRouteMatch = $current_route_match;
    $this->matcher = $matcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    /*  Check if redirection needs to happen after path and route
     *  have been determined.
     */
    $events[KernelEvents::CONTROLLER][] = ['onKernelRouteDeterminedDomain', -50];
    return $events;
  }

  /**
   * Decides whether to redirect depending on the identified route.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterControllerEvent $event
   *   The Event to process.
   */
  public function onKernelRouteDeterminedDomain(FilterControllerEvent $event) {
    if (!$event->isMasterRequest()) {
      return;
    }
    // Negotiate the request and set domain context.
    if ($active_domain = $this->domainNegotiator->getActiveDomain(TRUE)) {
      // Find matching domain_route_redirect if exists.
      $current_path = $event->getRequest()->getRequestUri();
      $current_route = $this->currentRouteMatch->getRouteName();

      if($domain_route_redirect = $this->matcher->find_match($active_domain, $current_path, $current_route)) {
          $domain_url = $domain_route_redirect->getDomainTarget()->getUrl();
          $redirect_code = $domain_route_redirect->getRedirect();

          if ($redirect_code) {
            $event->stopPropagation();
            $event->setController(function () use ($domain_url, $redirect_code) {
                return new TrustedRedirectResponse($domain_url, $redirect_code);
            });
          }
      }
    }
  }

}
