<?php

namespace Drupal\domain_route_redirect\HttpKernel;

use Drupal\Core\PathProcessor\OutboundPathProcessorInterface;

use Drupal\domain\DomainNegotiatorInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\domain_route_redirect\DomainRouteRedirectMatcherService;

use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Url;

/**
 * Processes the outbound path using path alias lookups.
 */
class DomainRouteRedirectPathProcessor implements OutboundPathProcessorInterface {

  /**
   * The Domain negotiator.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface
   */
  protected $negotiator;

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The domain_route_redirect matcher service.
   *
   * @var \Drupal\domain_route_redirect\DomainRouteRedirectMatcherService
   */
  protected $matcher;

  /**
   * The active domain request.
   *
   * @var \Drupal\domain\DomainInterface
   */
  protected $activeDomain;

  /**
   * Constructs a DomainSourcePathProcessor object.
   *
   * @param \Drupal\domain\DomainNegotiatorInterface $negotiator
   *   The domain negotiator.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   * @param Drupal\domain_route_redirect\DomainRouteRedirectMatcherService $matcher
   *   The domain_route_redirect matcher service.
   */
  public function __construct(DomainNegotiatorInterface $negotiator, AliasManagerInterface $alias_manager, DomainRouteRedirectMatcherService $matcher) {
    $this->negotiator = $negotiator;
    $this->aliasManager = $alias_manager;
    $this->matcher = $matcher;
  }

  /**
   * {@inheritdoc}
   */
  public function processOutbound($path, &$options = [], Request $request = NULL, BubbleableMetadata $bubbleable_metadata = NULL) {
    // Load the active domain if not set.
    if (empty($options['active_domain'])) {
      $active_domain = $this->getActiveDomain();
    }

    // Only act on valid internal paths and when a domain loads.
    if (empty($active_domain) || empty($path) || !empty($options['external'])) {
      return $path;
    }

    // Set the default source information.
    $options['active_domain'] = $active_domain;

    // Get the current language.
    $langcode = NULL;
    if (!empty($options['language'])) {
      $langcode = $options['language']->getId();
    }

    // Get the URL object for this request.
    $alias = $this->aliasManager->getPathByAlias($path, $langcode);
    $url = Url::fromUserInput($alias, $options);

    // Set current path and current route.
    if ($url->isRouted()) {
      $route = $url->getRouteName();
    }
    else {
      $route = '';
    }

    // Find matching domain_route_redirect if exists.
    if($domain_route_redirect = $this->matcher->find_match($active_domain, $path, $route)) {
        $domain_url = $domain_route_redirect->getDomainTarget()->getPath();
        $redirect_code = $domain_route_redirect->getRedirect();

        if ($redirect_code) {
          $options['base_url'] = trim($domain_url, '/');
          $options['absolute'] = TRUE;
        }
    }

    return $path;
  }

  /**
   * Gets the active domain.
   *
   * @return \Drupal\domain\DomainInterface
   *   The active domain.
   */
  public function getActiveDomain() {
    if (!isset($this->activeDomain)) {
      // Ensure that the loader has run.
      // In some tests, the kernel event has not.
      $active = $this->negotiator->getActiveDomain();
      if (empty($active)) {
        $active = $this->negotiator->getActiveDomain(TRUE);
      }
      $this->activeDomain = $active;
    }
    return $this->activeDomain;
  }

}
